use rand::{prelude::StdRng, Rng, SeedableRng};//(1)
use std::ops::Range;

pub struct RandomNumberGenerator {//(2)
  rng: StdRng,//(3)
}

impl Default for RandomNumberGenerator {
  fn default() -> Self {
    Self::new()
  }
}

impl RandomNumberGenerator {
  pub fn new() -> Self {//(4)
    Self {
      rng: StdRng::from_entropy(),//(5)
    }
  }

  pub fn seeded(seed: u64) -> Self {//(6)
    Self {
      rng: StdRng::seed_from_u64(seed),
    }
  }

  pub fn range(&mut self, range: Range<u32>) -> u32 {
    self.rng.gen_range(range)
  }
}

#[cfg(test)]
mod test {
  use super::*;

  #[test]//(7)
  fn test_range_bounds() {
    let mut rng = RandomNumberGenerator::new();
    for _ in 0..1000 {//(8)
      let n = rng.range(1..10);
      assert!(n >= 1);//(9)
      assert!(n < 10);//(10)
    }
  }

  #[test]
  fn test_reproducibility() {
    let mut rng = (
      RandomNumberGenerator::seeded(1),
      RandomNumberGenerator::seeded(1),
    );//(11)
    (0..1000).for_each(|_| {//(12)
      assert_eq!(//(13)
        rng.0.range(u32::MIN..u32::MAX),
        rng.1.range(u32::MIN..u32::MAX),
      );
    });
  }
}
