use bevy::{asset::{Asset, LoadedUntypedAsset}, prelude::*, utils::HashMap};

pub type LoadedAssets = Assets<LoadedUntypedAsset>;
pub type AssetResource<'w> = Res<'w, LoadedAssets>;

#[derive(Resource)]//(1)
pub struct AssetStore {
  pub(crate) asset_index: HashMap<String, Handle<LoadedUntypedAsset>>,//(2)
}

impl AssetStore {
  pub fn get_handle<T>(&self, index: &str, assets: &LoadedAssets) -> Option<Handle<T>>//(3)
  where
    T: Asset,//(4)
  {
    if let Some(handle_untyped) = self.asset_index.get(index) {//(5)
      if let Some(handle) = assets.get(handle_untyped) {//(6)
        return Some(handle.handle.clone().typed::<T>());//(7)
      }
      None
    } else {
      None//(8)
    }
  }

  pub fn play(&self, 
    sound_name: &str, 
    commands: &mut Commands, 
    assets: &LoadedAssets
  ) {
    let sound_handle: Handle<AudioSource> = self.get_handle(
      sound_name, 
      assets)
      .unwrap();
    commands.spawn((
        AudioBundle {
            source: sound_handle,
            ..default()
        },
    ));
  }
}
